@@ -29,6 +29,7 @@ class HttpMediaSession:
         self,
         ip: str,
         cloud_password: str,
+        super_secret_key: str,
         window_size=50,
         port: int = 8800,
         username: str = "admin",
@@ -37,6 +38,7 @@ class HttpMediaSession:
         self.ip = ip
         self.window_size = window_size
         self.cloud_password = cloud_password
+        self.super_secret_key = super_secret_key
         self.hashed_password = md5digest(cloud_password.encode()).decode()
         self.port = port
         self.username = username
@@ -174,7 +176,9 @@ class HttpMediaSession:
             # Prepare for AES decryption of content
             self._key_exchange = res_headers["Key-Exchange"]
             self._aes = AESHelper.from_keyexchange_and_password(
-                self._key_exchange.encode(), self.cloud_password.encode()
+                self._key_exchange.encode(),
+                self.cloud_password.encode(),
+                self.super_secret_key.encode(),
             )
 
             logger.debug("AES key exchange performed")
@@ -220,9 +224,12 @@ class HttpMediaSession:
 
             logger.debug("Handling new server response")
 
+            # print("got response")
+
             # Read and parse headers
             headers_block = await self._reader.readuntil(b"\r\n\r\n")
             headers = parse_http_headers(headers_block)
+            # print(headers)
 
             mimetype = headers["Content-Type"]
             length = int(headers["Content-Length"])
@@ -235,12 +242,22 @@ class HttpMediaSession:
 
             # Now we know the content length, let's read it and decrypt it
             json_data = None
+            # print("TEST0")
             data = await self._reader.readexactly(length)
             if encrypted:
+                # print("encrypted")
                 ciphertext = data
+                # print("TEST1")
                 try:
+                    # print("lolo")
+                    # print(ciphertext)
                     plaintext = self._aes.decrypt(ciphertext)
+                    # if length == 384:
+                    # print(plaintext)
+                    # print("lala")
+                    # print(plaintext)
                 except ValueError as e:
+                    # print(e)
                     if "padding is incorrect" in e.args[0].lower():
                         e = ValueError(
                             e.args[0]
@@ -251,18 +268,21 @@ class HttpMediaSession:
                 except Exception as e:
                     plaintext = e
             else:
+                # print("plaintext")
                 ciphertext = None
                 plaintext = data
-
+            # print(plaintext)
             # JSON responses sometimes have the above info in the payload,
             # not the headers. Let's parse it.
             if mimetype == "application/json":
                 try:
                     json_data = json.loads(plaintext.decode())
                     if "seq" in json_data:
+                        # print("Setting seq")
                         seq = json_data["seq"]
                     if "params" in json_data and "session_id" in json_data["params"]:
                         session = int(json_data["params"]["session_id"])
+                        # print("Setting session")
                 except JSONDecodeError:
                     logger.warning("Unable to parse JSON sent from device")
 
@@ -314,7 +334,12 @@ class HttpMediaSession:
                 json_data=json_data,
             )
 
-            if seq % self.window_size == 0 and seq < 2000:  # seq < 2000 is temp
+            if (
+                seq is not None  # never ack live stream
+                and seq % self.window_size == 0
+                and (seq < 2000)
+            ):  # seq < 2000 is temp
+                # print("sending ack")
                 data = {
                     "type": "notification",
                     "params": {"event_type": "stream_sequence"},
@@ -331,6 +356,7 @@ class HttpMediaSession:
                 await self._send_http_request(b"--" + self.client_boundary, headers)
                 chunk_size = 4096
                 for i in range(0, len(data), chunk_size):
+                    # print(data[i : i + chunk_size])
                     self._writer.write(data[i : i + chunk_size])
                     await self._writer.drain()
 
@@ -417,7 +443,9 @@ class HttpMediaSession:
         await self._send_http_request(b"--" + self.client_boundary, headers)
 
         chunk_size = 4096
+        # print("Sending:")
         for i in range(0, len(data), chunk_size):
+            # print(data[i : i + chunk_size])
             self._writer.write(data[i : i + chunk_size])
             await self._writer.drain()
 
@@ -462,6 +490,7 @@ class HttpMediaSession:
                     session = resp.session
                 if resp.encrypted and isinstance(resp.plaintext, Exception):
                     raise resp.plaintext
+                # print(resp.plaintext)
                 yield resp
 
         finally:
